@@ -1,4 +1,4 @@
-/* $Id: economy.cpp 16423 2009-05-25 17:15:15Z rubidium $ */
+/* $Id: economy.cpp 17009 2009-07-31 22:23:19Z rubidium $ */
 
 /** @file economy.cpp Handling of the economy. */
 
@@ -32,10 +32,16 @@
 #include "autoreplace_func.h"
 #include "company_gui.h"
 #include "signs_base.h"
+#include "economy_base.h"
+#include "oldpool_func.h"
 
 #include "table/strings.h"
 #include "table/sprites.h"
 
+
+/* Initialize the cargo payment-pool */
+DEFINE_OLD_POOL_GENERIC(CargoPayment, CargoPayment)
+
 /**
  * Multiply two integer values and shift the results to right.
  *
@@ -1220,15 +1226,17 @@ static bool FindIndustryToDeliver(TileIndex ind_tile, void *user_data)
 	return true;
 }
 
+/** The industries we've currently brought cargo to. */
+static SmallIndustryList _cargo_delivery_destinations;
+
 /**
  * Transfer goods from station to industry.
  * All cargo is delivered to the nearest (Manhattan) industry to the station sign, which is inside the acceptance rectangle and actually accepts the cargo.
  * @param st The station that accepted the cargo
  * @param cargo_type Type of cargo delivered
  * @param nun_pieces Amount of cargo delivered
- * @param industry_set The destination industry will be inserted into this set
  */
-static void DeliverGoodsToIndustry(const Station *st, CargoID cargo_type, int num_pieces, SmallIndustryList *industry_set)
+static void DeliverGoodsToIndustry(const Station *st, CargoID cargo_type, int num_pieces)
 {
 	if (st->rect.IsEmpty()) return;
 
@@ -1267,13 +1275,14 @@ static void DeliverGoodsToIndustry(const Station *st, CargoID cargo_type, int nu
 		assert(best != NULL);
 
 		/* Insert the industry into industry_set, if not yet contained */
-		if (industry_set != NULL) industry_set->Include(best);
+		_cargo_delivery_destinations.Include(best);
 
 		best->incoming_cargo_waiting[accepted_cargo_index] = min(num_pieces + best->incoming_cargo_waiting[accepted_cargo_index], 0xFFFF);
 	}
 }
 
-static bool CheckSubsidised(Station *from, Station *to, CargoID cargo_type)
+
+static bool CheckSubsidised(Station *from, Station *to, CargoID cargo_type, CompanyID company)
 {
 	Subsidy *s;
 	TileIndex xy;
@@ -1325,7 +1334,7 @@ static bool CheckSubsidised(Station *from, Station *to, CargoID cargo_type)
 			pair = SetupSubsidyDecodeParam(s, 0);
 			InjectDParam(1);
 
-			SetDParam(0, _current_company);
+			SetDParam(0, company);
 			AddNewsItem(
 				STR_2031_SERVICE_SUBSIDY_AWARDED + _settings_game.difficulty.subsidy_multiplier,
 				NS_SUBSIDIES,
@@ -1347,10 +1356,10 @@ static bool CheckSubsidised(Station *from, Station *to, CargoID cargo_type)
  * @param dest Station the cargo has been unloaded
  * @param source_tile The origin of the cargo for distance calculation
  * @param days_in_transit Travel time
- * @param industry_set The delivered industry will be inserted into this set, if not yet contained
+ * @param company The company delivering the cargo
  * The cargo is just added to the stockpile of the industry. It is due to the caller to trigger the industry's production machinery
  */
-static Money DeliverGoods(int num_pieces, CargoID cargo_type, StationID source, StationID dest, TileIndex source_tile, byte days_in_transit, SmallIndustryList *industry_set)
+static Money DeliverGoods(int num_pieces, CargoID cargo_type, StationID source, StationID dest, TileIndex source_tile, byte days_in_transit, Company *company)
 {
 	bool subsidised;
 	Station *s_from, *s_to;
@@ -1359,18 +1368,15 @@ static Money DeliverGoods(int num_pieces, CargoID cargo_type, StationID source,
 	assert(num_pieces > 0);
 
 	/* Update company statistics */
-	{
-		Company *c = GetCompany(_current_company);
-		c->cur_economy.delivered_cargo += num_pieces;
-		SetBit(c->cargo_types, cargo_type);
-	}
+	company->cur_economy.delivered_cargo += num_pieces;
+	SetBit(company->cargo_types, cargo_type);
 
 	/* Get station pointers. */
 	s_from = IsValidStationID(source) ? GetStation(source) : NULL;
 	s_to = GetStation(dest);
 
 	/* Check if a subsidy applies. */
-	subsidised = s_from != NULL && CheckSubsidised(s_from, s_to, cargo_type);
+	subsidised = s_from != NULL && CheckSubsidised(s_from, s_to, cargo_type, company->index);
 
 	/* Increase town's counter for some special goods types */
 	const CargoSpec *cs = GetCargo(cargo_type);
@@ -1378,7 +1384,7 @@ static Money DeliverGoods(int num_pieces, CargoID cargo_type, StationID source,
 	if (cs->town_effect == TE_WATER) s_to->town->new_act_water += num_pieces;
 
 	/* Give the goods to the industry. */
-	DeliverGoodsToIndustry(s_to, cargo_type, num_pieces, industry_set);
+	DeliverGoodsToIndustry(s_to, cargo_type, num_pieces);
 
 	/* Determine profit */
 	profit = GetTransportedGoodsIncome(num_pieces, DistanceManhattan(source_tile, s_to->xy), days_in_transit, cargo_type);
@@ -1432,109 +1438,107 @@ static void TriggerIndustryProduction(Industry *i)
 }
 
 /**
- * Performs the vehicle payment _and_ marks the vehicle to be unloaded.
+ * Makes us a new cargo payment helper.
+ * @param front The front of the train
+ * @param destinations List to add the destinations of 'our' cargo to
+ */
+CargoPayment::CargoPayment(Vehicle *front) :
+	front(front),
+	current_station(front->last_station_visited)
+{
+}
+
+CargoPayment::~CargoPayment()
+{
+	if (this->CleaningPool()) return;
+
+	this->front->cargo_payment = NULL;
+
+	if (this->visual_profit == 0) {
+		this->front = NULL;
+		return;
+	}
+
+	CompanyID old_company = _current_company;
+	_current_company = this->front->owner;
+
+	SubtractMoneyFromCompany(CommandCost(this->front->GetExpenseType(true), -this->route_profit));
+	this->front->profit_this_year += this->visual_profit << 8;
+
+	if (this->route_profit != 0) {
+		if (IsLocalCompany() && !PlayVehicleSound(this->front, VSE_LOAD_UNLOAD)) {
+			SndPlayVehicleFx(SND_14_CASHTILL, this->front);
+		}
+
+		ShowCostOrIncomeAnimation(this->front->x_pos, this->front->y_pos, this->front->z_pos, -this->visual_profit);
+	} else {
+		ShowFeederIncomeAnimation(this->front->x_pos, this->front->y_pos, this->front->z_pos, this->visual_profit);
+	}
+
+	_current_company = old_company;
+
+	this->front = NULL;
+}
+
+/**
+ * Handle payment for final delivery of the given cargo packet.
+ * @param cp The cargo packet to pay for.
+ * @param count The number of packets to pay for.
+ */
+void CargoPayment::PayFinalDelivery(CargoPacket *cp, uint count)
+{
+	if (this->owner == NULL) {
+		this->owner = GetCompany(this->front->owner);
+	}
+
+	/* Handle end of route payment */
+	Money profit = DeliverGoods(count, this->ct, cp->source, this->current_station, cp->source_xy, cp->days_in_transit, this->owner);
+	this->route_profit += profit;
+
+	/* The vehicle's profit is whatever route profit there is minus feeder shares. */
+	this->visual_profit += profit - cp->feeder_share;
+}
+
+/**
+ * Handle payment for transfer of the given cargo packet.
+ * @param cp The cargo packet to pay for.
+ * @param count The number of packets to pay for.
+ */
+void CargoPayment::PayTransfer(CargoPacket *cp, uint count)
+{
+	Money profit = GetTransportedGoodsIncome(
+		count,
+		/* pay transfer vehicle for only the part of transfer it has done: ie. cargo_loaded_at_xy to here */
+		DistanceManhattan(cp->loaded_at_xy, GetStation(this->current_station)->xy),
+		cp->days_in_transit,
+		this->ct);
+
+	this->visual_profit += profit; // accumulate transfer profits for whole vehicle
+	cp->feeder_share    += profit; // account for the (virtual) profit already made for the cargo packet
+}
+
+/**
+ * Prepare the vehicle to be unloaded.
  * @param front_v the vehicle to be unloaded
  */
-void VehiclePayment(Vehicle *front_v)
+void PrepareUnload(Vehicle *front_v)
 {
-	int result = 0;
-
-	Money vehicle_profit = 0; // Money paid to the train
-	Money route_profit   = 0; // The grand total amount for the route. A-D of transfer chain A-B-C-D
-	Money virtual_profit = 0; // The virtual profit for entire vehicle chain
-
-	StationID last_visited = front_v->last_station_visited;
-	Station *st = GetStation(last_visited);
-
-	/* The owner of the train wants to be paid */
-	CompanyID old_company = _current_company;
-	_current_company = front_v->owner;
-
 	/* At this moment loading cannot be finished */
 	ClrBit(front_v->vehicle_flags, VF_LOADING_FINISHED);
 
 	/* Start unloading in at the first possible moment */
 	front_v->load_unload_time_rem = 1;
 
-	/* Collect delivered industries */
-	static SmallIndustryList industry_set;
-	industry_set.Clear();
-
-	for (Vehicle *v = front_v; v != NULL; v = v->Next()) {
-		/* No cargo to unload */
-		if (v->cargo_cap == 0 || v->cargo.Empty() || front_v->current_order.GetUnloadType() & OUFB_NO_UNLOAD) continue;
-
-		/* All cargo has already been paid for, no need to pay again */
-		if (!v->cargo.UnpaidCargo()) {
-			SetBit(v->vehicle_flags, VF_CARGO_UNLOADING);
-			continue;
-		}
-
-		GoodsEntry *ge = &st->goods[v->cargo_type];
-		const CargoList::List *cargos = v->cargo.Packets();
-
-		for (CargoList::List::const_iterator it = cargos->begin(); it != cargos->end(); it++) {
-			CargoPacket *cp = *it;
-			if (!cp->paid_for &&
-					cp->source != last_visited &&
-					HasBit(ge->acceptance_pickup, GoodsEntry::ACCEPTANCE) &&
-					(front_v->current_order.GetUnloadType() & OUFB_TRANSFER) == 0) {
-				/* Deliver goods to the station */
-				st->time_since_unload = 0;
-
-				/* handle end of route payment */
-				Money profit = DeliverGoods(cp->count, v->cargo_type, cp->source, last_visited, cp->source_xy, cp->days_in_transit, &industry_set);
-				cp->paid_for = true;
-				route_profit   += profit; // display amount paid for final route delivery, A-D of a chain A-B-C-D
-				vehicle_profit += profit - cp->feeder_share;                    // whole vehicle is not payed for transfers picked up earlier
-
-				result |= 1;
-
-				SetBit(v->vehicle_flags, VF_CARGO_UNLOADING);
-			} else if (front_v->current_order.GetUnloadType() & (OUFB_UNLOAD | OUFB_TRANSFER)) {
-				if (!cp->paid_for && (front_v->current_order.GetUnloadType() & OUFB_TRANSFER) != 0) {
-					Money profit = GetTransportedGoodsIncome(
-						cp->count,
-						/* pay transfer vehicle for only the part of transfer it has done: ie. cargo_loaded_at_xy to here */
-						DistanceManhattan(cp->loaded_at_xy, GetStation(last_visited)->xy),
-						cp->days_in_transit,
-						v->cargo_type);
-
-					front_v->profit_this_year += profit << 8;
-					virtual_profit   += profit; // accumulate transfer profits for whole vehicle
-					cp->feeder_share += profit; // account for the (virtual) profit already made for the cargo packet
-					cp->paid_for      = true;   // record that the cargo has been paid for to eliminate double counting
-				}
-				result |= 2;
-
+	if ((front_v->current_order.GetUnloadType() & OUFB_NO_UNLOAD) == 0) {
+		for (Vehicle *v = front_v; v != NULL; v = v->Next()) {
+			if (v->cargo_cap > 0 && !v->cargo.Empty()) {
 				SetBit(v->vehicle_flags, VF_CARGO_UNLOADING);
 			}
 		}
-		v->cargo.InvalidateCache();
 	}
 
-	/* Call the production machinery of industries only once for every vehicle chain */
-	const Industry * const *isend = industry_set.End();
-	for (Industry **iid = industry_set.Begin(); iid != isend; iid++) {
-		TriggerIndustryProduction(*iid);
-	}
-
-	if (virtual_profit > 0) {
-		ShowFeederIncomeAnimation(front_v->x_pos, front_v->y_pos, front_v->z_pos, virtual_profit);
-	}
-
-	if (route_profit != 0) {
-		front_v->profit_this_year += vehicle_profit << 8;
-		SubtractMoneyFromCompany(CommandCost(front_v->GetExpenseType(true), -route_profit));
-
-		if (IsLocalCompany() && !PlayVehicleSound(front_v, VSE_LOAD_UNLOAD)) {
-			SndPlayVehicleFx(SND_14_CASHTILL, front_v);
-		}
-
-		ShowCostOrIncomeAnimation(front_v->x_pos, front_v->y_pos, front_v->z_pos, -vehicle_profit);
-	}
-
-	_current_company = old_company;
+	assert(front_v->cargo_payment == NULL);
+	front_v->cargo_payment = new CargoPayment(front_v);
 }
 
 /**
@@ -1548,6 +1552,7 @@ void VehiclePayment(Vehicle *front_v)
 static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 {
 	assert(v->current_order.IsType(OT_LOADING));
+	assert(v->load_unload_time_rem != 0);
 
 	/* We have not waited enough time till the next round of loading/unloading */
 	if (--v->load_unload_time_rem != 0) {
@@ -1583,6 +1588,8 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 
 	v->cur_speed = 0;
 
+	CargoPayment *payment = v->cargo_payment;
+
 	for (; v != NULL; v = v->Next()) {
 		if (v->cargo_cap == 0) continue;
 
@@ -1604,9 +1611,11 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 			bool remaining = false; // Are there cargo entities in this vehicle that can still be unloaded here?
 			bool accepted  = false; // Is the cargo accepted by the station?
 
+			payment->SetCargo(v->cargo_type);
+
 			if (HasBit(ge->acceptance_pickup, GoodsEntry::ACCEPTANCE) && !(u->current_order.GetUnloadType() & OUFB_TRANSFER)) {
 				/* The cargo has reached it's final destination, the packets may now be destroyed */
-				remaining = v->cargo.MoveTo(NULL, amount_unloaded, CargoList::MTA_FINAL_DELIVERY, last_visited);
+				remaining = v->cargo.MoveTo(NULL, amount_unloaded, CargoList::MTA_FINAL_DELIVERY, payment, last_visited);
 
 				result |= 1;
 				accepted = true;
@@ -1618,14 +1627,20 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 			 * station is still accepting the cargo in the vehicle. It doesn't
 			 * accept cargo that was loaded at the same station. */
 			if (u->current_order.GetUnloadType() & (OUFB_UNLOAD | OUFB_TRANSFER) && (!accepted || v->cargo.Count() == cargo_count)) {
-				remaining = v->cargo.MoveTo(&ge->cargo, amount_unloaded);
+				remaining = v->cargo.MoveTo(&ge->cargo, amount_unloaded, u->current_order.GetUnloadType() & OUFB_TRANSFER ? CargoList::MTA_TRANSFER : CargoList::MTA_UNLOAD, payment);
 				SetBit(ge->acceptance_pickup, GoodsEntry::PICKUP);
 
 				result |= 2;
 			} else if (!accepted) {
 				/* The order changed while unloading (unset unload/transfer) or the
-				 * station does not accept goods anymore. */
+				 * station does not accept our goods. */
 				ClrBit(v->vehicle_flags, VF_CARGO_UNLOADING);
+
+				/* Say we loaded something, otherwise we'll think we didn't unload
+				 * something and we didn't load something, so we must be finished
+				 * at this station. Setting the unloaded means that we will get a
+				 * retry for loading in the next cycle. */
+				anything_unloaded = true;
 				continue;
 			}
 
@@ -1695,7 +1710,7 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 			completely_emptied = false;
 			anything_loaded = true;
 
-			ge->cargo.MoveTo(&v->cargo, cap, CargoList::MTA_CARGO_LOAD, st->xy);
+			ge->cargo.MoveTo(&v->cargo, cap, CargoList::MTA_CARGO_LOAD, NULL, st->xy);
 
 			st->time_since_load = 0;
 			st->last_vehicle_type = v->type;
@@ -1731,6 +1746,8 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 
 	v = u;
 
+	if (!anything_unloaded) delete payment;
+
 	if (anything_loaded || anything_unloaded) {
 		if (_settings_game.order.gradual_loading) {
 			/* The time it takes to load one 'slice' of cargo or passengers depends
@@ -1783,7 +1800,8 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
 		}
 	}
 
-	v->load_unload_time_rem = unloading_time;
+	/* Always wait at least 1, otherwise we'll wait 'infinitively' long. */
+	v->load_unload_time_rem = max(1, unloading_time);
 
 	if (completely_emptied) {
 		TriggerVehicle(v, VEHICLE_TRIGGER_EMPTY);
@@ -1807,6 +1825,9 @@ static void LoadUnloadVehicle(Vehicle *v, int *cargo_left)
  */
 void LoadUnloadStation(Station *st)
 {
+	/* No vehicle is here... */
+	if (st->loading_vehicles.empty()) return;
+
 	int cargo_left[NUM_CARGO];
 
 	for (uint i = 0; i < NUM_CARGO; i++) cargo_left[i] = st->goods[i].cargo.Count();
@@ -1816,6 +1837,13 @@ void LoadUnloadStation(Station *st)
 		Vehicle *v = *iter;
 		if (!(v->vehstatus & (VS_STOPPED | VS_CRASHED))) LoadUnloadVehicle(v, cargo_left);
 	}
+
+	/* Call the production machinery of industries */
+	const Industry * const *isend = _cargo_delivery_destinations.End();
+	for (Industry **iid = _cargo_delivery_destinations.Begin(); iid != isend; iid++) {
+		TriggerIndustryProduction(*iid);
+	}
+	_cargo_delivery_destinations.Clear();
 }
 
 void CompaniesMonthlyLoop()
