@@ -1,4 +1,4 @@
-/* $Id: road_map.h 15711 2009-03-14 18:16:29Z rubidium $ */
+/* $Id: road_map.h 16804 2009-07-13 10:16:50Z rubidium $ */
 
 /** @file road_map.h Map accessors for roads. */
 
@@ -361,16 +361,6 @@ static inline DiagDirection GetRoadDepotDirection(TileIndex t)
  */
 RoadBits GetAnyRoadBits(TileIndex tile, RoadType rt, bool straight_tunnel_bridge_entrance = false);
 
-/**
- * Get the accessible track bits for the given tile.
- * Special behaviour:
- *  - road depots: no track bits
- *  - non-drive-through stations: no track bits
- * @param tile the tile to get the track bits for
- * @return the track bits for the given tile
- */
-TrackBits GetAnyRoadTrackBits(TileIndex tile, RoadType rt);
-
 /**
  * Return if the tile is a valid tile for a crossing.
  *
