@@ -1,9 +1,13 @@
-/* $Id: strgen.cpp 12478 2008-03-28 19:01:57Z rubidium $ */
+/* $Id: strgen.cpp 16830 2009-07-14 20:38:45Z rubidium $ */
+
+/** @file strgen.cpp Tool to create computer readable (stand-alone) translation files. */
 
 #include "../stdafx.h"
 #include "../core/alloc_func.hpp"
 #include "../core/endian_func.hpp"
 #include "../string_func.h"
+#include "../strings_type.h"
+#include "strgen.h"
 #include "../table/control_codes.h"
 
 #include <stdio.h>
@@ -32,17 +36,6 @@
 
 typedef void (*ParseCmdProc)(char *buf, int value);
 
-struct LanguagePackHeader {
-	uint32 ident;
-	uint32 version;     // 32-bits of auto generated version info which is basically a hash of strings.h
-	char name[32];      // the international name of this language
-	char own_name[32];  // the localized name of this language
-	char isocode[16];   // the ISO code for the language (not country code)
-	uint16 offsets[32]; // the offsets
-	byte plural_form;   // plural form index
-	byte pad[3];        // pad header to be a multiple of 4
-};
-
 struct CmdStruct {
 	const char *cmd;
 	ParseCmdProc proc;
@@ -65,7 +58,7 @@ struct Case {
 
 static bool _masterlang;
 static bool _translated;
-static const char* _file = "(unknown file)";
+static const char *_file = "(unknown file)";
 static int _cur_line;
 static int _errors, _warnings, _show_todo;
 
@@ -93,17 +86,20 @@ static int _next_string_id;
 static uint32 _hash;
 static char _lang_name[32], _lang_ownname[32], _lang_isocode[16];
 static byte _lang_pluralform;
+static byte _lang_textdir;
+static uint16 _lang_winlangid;
+static uint8 _lang_newgrflangid;
 #define MAX_NUM_GENDER 8
-static char _genders[MAX_NUM_GENDER][8];
-static int _numgenders;
+static char _genders[MAX_NUM_GENDER][16];
+static uint _numgenders;
 
-// contains the name of all cases.
+/* contains the name of all cases. */
 #define MAX_NUM_CASES 50
 static char _cases[MAX_NUM_CASES][16];
-static int _numcases;
+static uint _numcases;
 
-// for each plural value, this is the number of plural forms.
-static const byte _plural_form_counts[] = { 2, 1, 2, 3, 3, 3, 3, 3, 4, 2 };
+/* for each plural value, this is the number of plural forms. */
+static const byte _plural_form_counts[] = { 2, 1, 2, 3, 3, 3, 3, 3, 4, 2, 3, 2 };
 
 static const char *_cur_ident;
 
@@ -113,12 +109,12 @@ struct CmdPair {
 };
 
 struct ParsedCommandStruct {
-	int np;
+	uint np;
 	CmdPair pairs[32];
 	const CmdStruct *cmd[32]; // ordered by param #
 };
 
-// Used when generating some advanced commands.
+/* Used when generating some advanced commands. */
 static ParsedCommandStruct _cur_pcs;
 static int _cur_argidx;
 
@@ -141,7 +137,7 @@ static LangString *HashFind(const char *s)
 	int idx = _hash_head[HashStr(s)];
 
 	while (--idx >= 0) {
-		LangString* ls = _strings[idx];
+		LangString *ls = _strings[idx];
 
 		if (strcmp(ls->name, s) == 0) return ls;
 		idx = ls->hash_next;
@@ -155,7 +151,9 @@ static LangString *HashFind(const char *s)
 # define LINE_NUM_FMT ":%d"
 #endif
 
-static void CDECL warning(const char *s, ...)
+static void CDECL strgen_warning(const char *s, ...) WARN_FORMAT(1, 2);
+
+static void CDECL strgen_warning(const char *s, ...)
 {
 	char buf[1024];
 	va_list va;
@@ -166,7 +164,9 @@ static void CDECL warning(const char *s, ...)
 	_warnings++;
 }
 
-void CDECL error(const char *s, ...)
+static void CDECL strgen_error(const char *s, ...) WARN_FORMAT(1, 2);
+
+static void CDECL strgen_error(const char *s, ...)
 {
 	char buf[1024];
 	va_list va;
@@ -177,8 +177,7 @@ void CDECL error(const char *s, ...)
 	_errors++;
 }
 
-
-static void NORETURN CDECL fatal(const char *s, ...)
+void NORETURN CDECL error(const char *s, ...)
 {
 	char buf[1024];
 	va_list va;
@@ -191,7 +190,7 @@ static void NORETURN CDECL fatal(const char *s, ...)
 
 static void PutByte(byte c)
 {
-	if (_put_pos == lengthof(_put_buf)) fatal("Put buffer too small");
+	if (_put_pos == lengthof(_put_buf)) error("Put buffer too small");
 	_put_buf[_put_pos++] = c;
 }
 
@@ -213,7 +212,7 @@ static void PutUtf8(uint32 value)
 		PutByte(0x80 + GB(value,  6, 6));
 		PutByte(0x80 + GB(value,  0, 6));
 	} else {
-		warning("Invalid unicode value U+0x%X", value);
+		strgen_warning("Invalid unicode value U+0x%X", value);
 	}
 }
 
@@ -245,7 +244,7 @@ size_t Utf8Validate(const char *s)
 
 static void EmitSingleChar(char *buf, int value)
 {
-	if (*buf != '\0') warning("Ignoring trailing letters in command");
+	if (*buf != '\0') strgen_warning("Ignoring trailing letters in command");
 	PutUtf8(value);
 }
 
@@ -254,7 +253,7 @@ static void EmitSetX(char *buf, int value)
 {
 	char *err;
 	int x = strtol(buf, &err, 0);
-	if (*err != 0) fatal("SetX param invalid");
+	if (*err != 0) error("SetX param invalid");
 	PutUtf8(SCC_SETX);
 	PutByte((byte)x);
 }
@@ -267,25 +266,25 @@ static void EmitSetXY(char *buf, int value)
 	int y;
 
 	x = strtol(buf, &err, 0);
-	if (*err != ' ') fatal("SetXY param invalid");
+	if (*err != ' ') error("SetXY param invalid");
 	y = strtol(err + 1, &err, 0);
-	if (*err != 0) fatal("SetXY param invalid");
+	if (*err != 0) error("SetXY param invalid");
 
 	PutUtf8(SCC_SETXY);
 	PutByte((byte)x);
 	PutByte((byte)y);
 }
 
-// The plural specifier looks like
-// {NUM} {PLURAL -1 passenger passengers} then it picks either passenger/passengers depending on the count in NUM
+/* The plural specifier looks like
+ * {NUM} {PLURAL -1 passenger passengers} then it picks either passenger/passengers depending on the count in NUM */
 
-// This is encoded like
-//  CommandByte <ARG#> <NUM> {Length of each string} {each string}
+/* This is encoded like
+ *  CommandByte <ARG#> <NUM> {Length of each string} {each string} */
 
 bool ParseRelNum(char **buf, int *value)
 {
-	const char* s = *buf;
-	char* end;
+	const char *s = *buf;
+	char *end;
 	bool rel = false;
 	int v;
 
@@ -305,7 +304,7 @@ bool ParseRelNum(char **buf, int *value)
 	return true;
 }
 
-// Parse out the next word, or NULL
+/* Parse out the next word, or NULL */
 char *ParseWord(char **buf)
 {
 	char *s = *buf, *r;
@@ -315,7 +314,7 @@ char *ParseWord(char **buf)
 
 	if (*s == '"') {
 		r = ++s;
-		// parse until next " or NUL
+		/* parse until next " or NUL */
 		for (;;) {
 			if (*s == '\0') break;
 			if (*s == '"') {
@@ -325,7 +324,7 @@ char *ParseWord(char **buf)
 			s++;
 		}
 	} else {
-		// proceed until whitespace or NUL
+		/* proceed until whitespace or NUL */
 		r = s;
 		for (;;) {
 			if (*s == '\0') break;
@@ -340,10 +339,10 @@ char *ParseWord(char **buf)
 	return r;
 }
 
-// Forward declaration
+/* Forward declaration */
 static int TranslateArgumentIdx(int arg);
 
-static void EmitWordList(const char* const* words, uint nw)
+static void EmitWordList(const char * const *words, uint nw)
 {
 	uint i;
 	uint j;
@@ -358,27 +357,27 @@ static void EmitWordList(const char* const* words, uint nw)
 static void EmitPlural(char *buf, int value)
 {
 	int argidx = _cur_argidx;
-	const char* words[5];
+	const char *words[5];
 	int nw = 0;
 
-	// Parse out the number, if one exists. Otherwise default to prev arg.
+	/* Parse out the number, if one exists. Otherwise default to prev arg. */
 	if (!ParseRelNum(&buf, &argidx)) argidx--;
 
-	// Parse each string
+	/* Parse each string */
 	for (nw = 0; nw < 5; nw++) {
 		words[nw] = ParseWord(&buf);
 		if (words[nw] == NULL) break;
 	}
 
 	if (nw == 0)
-		fatal("%s: No plural words", _cur_ident);
+		error("%s: No plural words", _cur_ident);
 
 	if (_plural_form_counts[_lang_pluralform] != nw) {
 		if (_translated) {
-			fatal("%s: Invalid number of plural forms. Expecting %d, found %d.", _cur_ident,
+			error("%s: Invalid number of plural forms. Expecting %d, found %d.", _cur_ident,
 				_plural_form_counts[_lang_pluralform], nw);
 		} else {
-			if ((_show_todo & 2) != 0) warning("'%s' is untranslated. Tweaking english string to allow compilation for plural forms", _cur_ident);
+			if ((_show_todo & 2) != 0) strgen_warning("'%s' is untranslated. Tweaking english string to allow compilation for plural forms", _cur_ident);
 			if (nw > _plural_form_counts[_lang_pluralform]) {
 				nw = _plural_form_counts[_lang_pluralform];
 			} else {
@@ -403,26 +402,26 @@ static void EmitGender(char *buf, int value)
 	if (buf[0] == '=') {
 		buf++;
 
-		// This is a {G=DER} command
+		/* This is a {G=DER} command */
 		for (nw = 0; ; nw++) {
-			if (nw >= 8) fatal("G argument '%s' invalid", buf);
+			if (nw >= 8) error("G argument '%s' invalid", buf);
 			if (strcmp(buf, _genders[nw]) == 0) break;
 		}
-		// now nw contains the gender index
+		/* now nw contains the gender index */
 		PutUtf8(SCC_GENDER_INDEX);
 		PutByte(nw);
 	} else {
-		const char* words[8];
+		const char *words[8];
 
-		// This is a {G 0 foo bar two} command.
-		// If no relative number exists, default to +0
+		/* This is a {G 0 foo bar two} command.
+		 * If no relative number exists, default to +0 */
 		if (!ParseRelNum(&buf, &argidx)) {}
 
 		for (nw = 0; nw < 8; nw++) {
 			words[nw] = ParseWord(&buf);
 			if (words[nw] == NULL) break;
 		}
-		if (nw != _numgenders) fatal("Bad # of arguments for gender command");
+		if (nw != _numgenders) error("Bad # of arguments for gender command");
 		PutUtf8(SCC_GENDER_LIST);
 		PutByte(TranslateArgumentIdx(argidx));
 		EmitWordList(words, nw);
@@ -431,15 +430,15 @@ static void EmitGender(char *buf, int value)
 
 
 static const CmdStruct _cmd_structs[] = {
-	// Update position
+	/* Update position */
 	{"SETX",  EmitSetX,  SCC_SETX,  0, 0},
 	{"SETXY", EmitSetXY, SCC_SETXY, 0, 0},
 
-	// Font size
+	/* Font size */
 	{"TINYFONT", EmitSingleChar, SCC_TINYFONT, 0, 0},
 	{"BIGFONT",  EmitSingleChar, SCC_BIGFONT,  0, 0},
 
-	// Colors
+	/* Colors */
 	{"BLUE",    EmitSingleChar, SCC_BLUE,    0, 0},
 	{"SILVER",  EmitSingleChar, SCC_SILVER,  0, 0},
 	{"GOLD",    EmitSingleChar, SCC_GOLD,    0, 0},
@@ -462,11 +461,11 @@ static const CmdStruct _cmd_structs[] = {
 	{"REV",           EmitSingleChar, SCC_REVISION,            0, 0}, // openttd revision string
 	{"SHORTCARGO",    EmitSingleChar, SCC_CARGO_SHORT,         2, 0}, // short cargo description, only ### tons, or ### litres
 
-	{"STRING1", EmitSingleChar, SCC_STRING1, 1, C_CASE}, // included string that consumes ONE argument
-	{"STRING2", EmitSingleChar, SCC_STRING2, 2, C_CASE}, // included string that consumes TWO arguments
-	{"STRING3", EmitSingleChar, SCC_STRING3, 3, C_CASE}, // included string that consumes THREE arguments
-	{"STRING4", EmitSingleChar, SCC_STRING4, 4, C_CASE}, // included string that consumes FOUR arguments
-	{"STRING5", EmitSingleChar, SCC_STRING5, 5, C_CASE}, // included string that consumes FIVE arguments
+	{"STRING1", EmitSingleChar, SCC_STRING1, 2, C_CASE}, // included string that consumes the string id and ONE argument
+	{"STRING2", EmitSingleChar, SCC_STRING2, 3, C_CASE}, // included string that consumes the string id and TWO arguments
+	{"STRING3", EmitSingleChar, SCC_STRING3, 4, C_CASE}, // included string that consumes the string id and THREE arguments
+	{"STRING4", EmitSingleChar, SCC_STRING4, 5, C_CASE}, // included string that consumes the string id and FOUR arguments
+	{"STRING5", EmitSingleChar, SCC_STRING5, 6, C_CASE}, // included string that consumes the string id and FIVE arguments
 
 	{"STATIONFEATURES", EmitSingleChar, SCC_STATION_FEATURES, 1, 0}, // station features string, icons of the features
 	{"INDUSTRY",        EmitSingleChar, SCC_INDUSTRY_NAME,    1, 0}, // industry, takes an industry #
@@ -485,14 +484,17 @@ static const CmdStruct _cmd_structs[] = {
 	{"DATE_TINY",  EmitSingleChar, SCC_DATE_TINY, 1, 0},
 	{"DATE_SHORT", EmitSingleChar, SCC_DATE_SHORT, 1, 0},
 	{"DATE_LONG",  EmitSingleChar, SCC_DATE_LONG, 1, 0},
+	{"DATE_ISO",   EmitSingleChar, SCC_DATE_ISO, 1, 0},
 
 	{"SKIP", EmitSingleChar, SCC_SKIP, 1, 0},
 
 	{"STRING", EmitSingleChar, SCC_STRING, 1, C_CASE},
+	{"RAW_STRING", EmitSingleChar, SCC_RAW_STRING_POINTER, 1, 0},
 
-	// Numbers
+	/* Numbers */
 	{"COMMA", EmitSingleChar, SCC_COMMA, 1, 0}, // Number with comma
 	{"NUM",   EmitSingleChar, SCC_NUM,   1, 0}, // Signed number
+	{"BYTES", EmitSingleChar, SCC_BYTES, 1, 0}, // Unsigned number with "bytes", i.e. "1.02 MiB or 123 KiB"
 
 	{"CURRENCY",   EmitSingleChar, SCC_CURRENCY,    1, 0},
 
@@ -505,7 +507,7 @@ static const CmdStruct _cmd_structs[] = {
 	{"VEHICLE",  EmitSingleChar, SCC_VEHICLE_NAME,  1, 0},
 	{"COMPANY",  EmitSingleChar, SCC_COMPANY_NAME,  1, 0},
 	{"COMPANYNUM", EmitSingleChar, SCC_COMPANY_NUM, 1, 0},
-	{"PLAYERNAME", EmitSingleChar, SCC_PLAYER_NAME, 1, 0},
+	{"PRESIDENTNAME", EmitSingleChar, SCC_PRESIDENT_NAME, 1, 0},
 
 	// 0x9D is used for the pseudo command SETCASE
 	// 0x9E is used for case switching
@@ -533,12 +535,22 @@ static const CmdStruct _cmd_structs[] = {
 	{"RIGHTARROW",     EmitSingleChar, SCC_RIGHTARROW,     0, C_DONTCOUNT},
 	{"SMALLLEFTARROW", EmitSingleChar, SCC_LESSTHAN,       0, C_DONTCOUNT},
 	{"SMALLRIGHTARROW",EmitSingleChar, SCC_GREATERTHAN,    0, C_DONTCOUNT},
+
+	/* The following are directional formatting codes used to get the RTL strings right:
+	 * http://www.unicode.org/unicode/reports/tr9/#Directional_Formatting_Codes */
+	{"LRM",            EmitSingleChar, 0x200E,             0, C_DONTCOUNT},
+	{"RLM",            EmitSingleChar, 0x200F,             0, C_DONTCOUNT},
+	{"LRE",            EmitSingleChar, 0x202A,             0, C_DONTCOUNT},
+	{"RLE",            EmitSingleChar, 0x202B,             0, C_DONTCOUNT},
+	{"LRO",            EmitSingleChar, 0x202D,             0, C_DONTCOUNT},
+	{"RLO",            EmitSingleChar, 0x202E,             0, C_DONTCOUNT},
+	{"PDF",            EmitSingleChar, 0x202C,             0, C_DONTCOUNT},
 };
 
 
 static const CmdStruct *FindCmd(const char *s, int len)
 {
-	const CmdStruct* cs;
+	const CmdStruct *cs;
 
 	for (cs = _cmd_structs; cs != endof(_cmd_structs); cs++) {
 		if (strncmp(cs->cmd, s, len) == 0 && cs->cmd[len] == '\0') return cs;
@@ -553,12 +565,12 @@ static uint ResolveCaseName(const char *str, uint len)
 	for (i = 0; i < MAX_NUM_CASES; i++) {
 		if (memcmp(_cases[i], str, len) == 0 && _cases[i][len] == 0) return i + 1;
 	}
-	fatal("Invalid case-name '%s'", str);
+	error("Invalid case-name '%s'", str);
 }
 
 
-// returns NULL on eof
-// else returns command struct
+/* returns NULL on eof
+ * else returns command struct */
 static const CmdStruct *ParseCommandString(const char **str, char *param, int *argno, int *casei)
 {
 	const char *s = *str, *start;
@@ -568,7 +580,7 @@ static const CmdStruct *ParseCommandString(const char **str, char *param, int *a
 	*argno = -1;
 	*casei = -1;
 
-	// Scan to the next command, exit if there's no next command.
+	/* Scan to the next command, exit if there's no next command. */
 	for (; *s != '{'; s++) {
 		if (*s == '\0') return NULL;
 	}
@@ -578,11 +590,11 @@ static const CmdStruct *ParseCommandString(const char **str, char *param, int *a
 		char *end;
 
 		*argno = strtoul(s, &end, 0);
-		if (*end != ':') fatal("missing arg #");
+		if (*end != ':') error("missing arg #");
 		s = end + 1;
 	}
 
-	// parse command name
+	/* parse command name */
 	start = s;
 	do {
 		c = *s++;
@@ -590,7 +602,7 @@ static const CmdStruct *ParseCommandString(const char **str, char *param, int *a
 
 	cmd = FindCmd(start, s - start - 1);
 	if (cmd == NULL) {
-		error("Undefined command '%.*s'", s - start - 1, start);
+		strgen_error("Undefined command '%.*s'", (int)(s - start - 1), start);
 		return NULL;
 	}
 
@@ -598,30 +610,30 @@ static const CmdStruct *ParseCommandString(const char **str, char *param, int *a
 		const char *casep = s;
 
 		if (!(cmd->flags & C_CASE))
-			fatal("Command '%s' can't have a case", cmd->cmd);
+			error("Command '%s' can't have a case", cmd->cmd);
 
 		do c = *s++; while (c != '}' && c != ' ' && c != '\0');
 		*casei = ResolveCaseName(casep, s - casep - 1);
 	}
 
 	if (c == '\0') {
-		error("Missing } from command '%s'", start);
+		strgen_error("Missing } from command '%s'", start);
 		return NULL;
 	}
 
 
 	if (c != '}') {
 		if (c == '=') s--;
-		// copy params
+		/* copy params */
 		start = s;
 		for (;;) {
 			c = *s++;
 			if (c == '}') break;
 			if (c == '\0') {
-				error("Missing } from command '%s'", start);
+				strgen_error("Missing } from command '%s'", start);
 				return NULL;
 			}
-			if (s - start == 250) fatal("param command too long");
+			if (s - start == 250) error("param command too long");
 			*param++ = c;
 		}
 	}
@@ -638,43 +650,65 @@ static void HandlePragma(char *str)
 	if (!memcmp(str, "id ", 3)) {
 		_next_string_id = strtoul(str + 3, NULL, 0);
 	} else if (!memcmp(str, "name ", 5)) {
-		ttd_strlcpy(_lang_name, str + 5, sizeof(_lang_name));
+		strecpy(_lang_name, str + 5, lastof(_lang_name));
 	} else if (!memcmp(str, "ownname ", 8)) {
-		ttd_strlcpy(_lang_ownname, str + 8, sizeof(_lang_ownname));
+		strecpy(_lang_ownname, str + 8, lastof(_lang_ownname));
 	} else if (!memcmp(str, "isocode ", 8)) {
-		ttd_strlcpy(_lang_isocode, str + 8, sizeof(_lang_isocode));
+		strecpy(_lang_isocode, str + 8, lastof(_lang_isocode));
 	} else if (!memcmp(str, "plural ", 7)) {
 		_lang_pluralform = atoi(str + 7);
 		if (_lang_pluralform >= lengthof(_plural_form_counts))
-			fatal("Invalid pluralform %d", _lang_pluralform);
+			error("Invalid pluralform %d", _lang_pluralform);
+	} else if (!memcmp(str, "textdir ", 8)) {
+		if (!memcmp(str + 8, "ltr", 3)) {
+			_lang_textdir = TD_LTR;
+		} else if (!memcmp(str + 8, "rtl", 3)) {
+			_lang_textdir = TD_RTL;
+		} else {
+			error("Invalid textdir %s", str + 8);
+		}
+	} else if (!memcmp(str, "winlangid ", 10)) {
+		const char *buf = str + 10;
+		long langid = strtol(buf, NULL, 16);
+		if (langid > UINT16_MAX || langid < 0) {
+			error("Invalid winlangid %s", buf);
+		}
+		_lang_winlangid = (uint16)langid;
+	} else if (!memcmp(str, "grflangid ", 10)) {
+		const char *buf = str + 10;
+		long langid = strtol(buf, NULL, 16);
+		if (langid >= 0x7F || langid < 0) {
+			error("Invalid grflangid %s", buf);
+		}
+		_lang_newgrflangid = (uint8)langid;
 	} else if (!memcmp(str, "gender ", 7)) {
-		char* buf = str + 7;
+		char *buf = str + 7;
 
 		for (;;) {
-			const char* s = ParseWord(&buf);
+			const char *s = ParseWord(&buf);
 
 			if (s == NULL) break;
-			if (_numgenders >= MAX_NUM_GENDER) fatal("Too many genders, max %d", MAX_NUM_GENDER);
-			ttd_strlcpy(_genders[_numgenders], s, sizeof(_genders[_numgenders]));
+			if (_numgenders >= MAX_NUM_GENDER) error("Too many genders, max %d", MAX_NUM_GENDER);
+			strecpy(_genders[_numgenders], s, lastof(_genders[_numgenders]));
 			_numgenders++;
 		}
 	} else if (!memcmp(str, "case ", 5)) {
-		char* buf = str + 5;
+		char *buf = str + 5;
 
 		for (;;) {
-			const char* s = ParseWord(&buf);
+			const char *s = ParseWord(&buf);
 
 			if (s == NULL) break;
-			if (_numcases >= MAX_NUM_CASES) fatal("Too many cases, max %d", MAX_NUM_CASES);
-			ttd_strlcpy(_cases[_numcases], s, sizeof(_cases[_numcases]));
+			if (_numcases >= MAX_NUM_CASES) error("Too many cases, max %d", MAX_NUM_CASES);
+			strecpy(_cases[_numcases], s, lastof(_cases[_numcases]));
 			_numcases++;
 		}
 	} else {
-		fatal("unknown pragma '%s'", str);
+		error("unknown pragma '%s'", str);
 	}
 }
 
-static void ExtractCommandString(ParsedCommandStruct* p, const char* s, bool warnings)
+static void ExtractCommandString(ParsedCommandStruct *p, const char *s, bool warnings)
 {
 	char param[100];
 	int argno;
@@ -684,22 +718,22 @@ static void ExtractCommandString(ParsedCommandStruct* p, const char* s, bool war
 	memset(p, 0, sizeof(*p));
 
 	for (;;) {
-		// read until next command from a.
-		const CmdStruct* ar = ParseCommandString(&s, param, &argno, &casei);
+		/* read until next command from a. */
+		const CmdStruct *ar = ParseCommandString(&s, param, &argno, &casei);
 
 		if (ar == NULL) break;
 
-		// Sanity checking
-		if (argno != -1 && ar->consumes == 0) fatal("Non consumer param can't have a paramindex");
+		/* Sanity checking */
+		if (argno != -1 && ar->consumes == 0) error("Non consumer param can't have a paramindex");
 
 		if (ar->consumes) {
 			if (argno != -1) argidx = argno;
-			if (argidx < 0 || argidx >= lengthof(p->cmd)) fatal("invalid param idx %d", argidx);
-			if (p->cmd[argidx] != NULL && p->cmd[argidx] != ar) fatal("duplicate param idx %d", argidx);
+			if (argidx < 0 || (uint)argidx >= lengthof(p->cmd)) error("invalid param idx %d", argidx);
+			if (p->cmd[argidx] != NULL && p->cmd[argidx] != ar) error("duplicate param idx %d", argidx);
 
 			p->cmd[argidx++] = ar;
 		} else if (!(ar->flags & C_DONTCOUNT)) { // Ignore some of them
-			if (p->np >= lengthof(p->pairs)) fatal("too many commands in string, max %d", lengthof(p->pairs));
+			if (p->np >= lengthof(p->pairs)) error("too many commands in string, max " PRINTF_SIZE, lengthof(p->pairs));
 			p->pairs[p->np].a = ar;
 			p->pairs[p->np].v = param[0] != '\0' ? strdup(param) : "";
 			p->np++;
@@ -716,7 +750,8 @@ static const CmdStruct *TranslateCmdForCompare(const CmdStruct *a)
 			strcmp(a->cmd, "STRING2") == 0 ||
 			strcmp(a->cmd, "STRING3") == 0 ||
 			strcmp(a->cmd, "STRING4") == 0 ||
-			strcmp(a->cmd, "STRING5") == 0) {
+			strcmp(a->cmd, "STRING5") == 0 ||
+			strcmp(a->cmd, "RAW_STRING") == 0){
 		return FindCmd("STRING", 6);
 	}
 
@@ -730,25 +765,25 @@ static bool CheckCommandsMatch(char *a, char *b, const char *name)
 {
 	ParsedCommandStruct templ;
 	ParsedCommandStruct lang;
-	int i, j;
+	uint i, j;
 	bool result = true;
 
 	ExtractCommandString(&templ, b, true);
 	ExtractCommandString(&lang, a, true);
 
-	// For each string in templ, see if we find it in lang
+	/* For each string in templ, see if we find it in lang */
 	if (templ.np != lang.np) {
-		warning("%s: template string and language string have a different # of commands", name);
+		strgen_warning("%s: template string and language string have a different # of commands", name);
 		result = false;
 	}
 
 	for (i = 0; i < templ.np; i++) {
-		// see if we find it in lang, and zero it out
+		/* see if we find it in lang, and zero it out */
 		bool found = false;
 		for (j = 0; j < lang.np; j++) {
 			if (templ.pairs[i].a == lang.pairs[j].a &&
 					strcmp(templ.pairs[i].v, lang.pairs[j].v) == 0) {
-				// it was found in both. zero it out from lang so we don't find it again
+				/* it was found in both. zero it out from lang so we don't find it again */
 				lang.pairs[j].a = NULL;
 				found = true;
 				break;
@@ -756,16 +791,16 @@ static bool CheckCommandsMatch(char *a, char *b, const char *name)
 		}
 
 		if (!found) {
-			warning("%s: command '%s' exists in template file but not in language file", name, templ.pairs[i].a->cmd);
+			strgen_warning("%s: command '%s' exists in template file but not in language file", name, templ.pairs[i].a->cmd);
 			result = false;
 		}
 	}
 
-	// if we reach here, all non consumer commands match up.
-	// Check if the non consumer commands match up also.
+	/* if we reach here, all non consumer commands match up.
+	 * Check if the non consumer commands match up also. */
 	for (i = 0; i < lengthof(templ.cmd); i++) {
 		if (TranslateCmdForCompare(templ.cmd[i]) != TranslateCmdForCompare(lang.cmd[i])) {
-			warning("%s: Param idx #%d '%s' doesn't match with template command '%s'", name, i,
+			strgen_warning("%s: Param idx #%d '%s' doesn't match with template command '%s'", name, i,
 				lang.cmd[i]  == NULL ? "<empty>" : lang.cmd[i]->cmd,
 				templ.cmd[i] == NULL ? "<empty>" : templ.cmd[i]->cmd);
 			result = false;
@@ -777,7 +812,7 @@ static bool CheckCommandsMatch(char *a, char *b, const char *name)
 
 static void HandleString(char *str, bool master)
 {
-	char *s,*t;
+	char *s, *t;
 	LangString *ent;
 	char *casep;
 
@@ -786,18 +821,18 @@ static void HandleString(char *str, bool master)
 		return;
 	}
 
-	// Ignore comments & blank lines
+	/* Ignore comments & blank lines */
 	if (*str == ';' || *str == ' ' || *str == '\0') return;
 
 	s = strchr(str, ':');
 	if (s == NULL) {
-		error("Line has no ':' delimiter");
+		strgen_error("Line has no ':' delimiter");
 		return;
 	}
 
-	// Trim spaces.
-	// After this str points to the command name, and s points to the command contents
-	for (t = s; t > str && (t[-1] == ' ' || t[-1] == '\t'); t--);
+	/* Trim spaces.
+	 * After this str points to the command name, and s points to the command contents */
+	for (t = s; t > str && (t[-1] == ' ' || t[-1] == '\t'); t--) {}
 	*t = 0;
 	s++;
 
@@ -806,37 +841,37 @@ static void HandleString(char *str, bool master)
 		const char *tmp;
 		for (tmp = s; *tmp != '\0';) {
 			size_t len = Utf8Validate(tmp);
-			if (len == 0) fatal("Invalid UTF-8 sequence in '%s'", s);
+			if (len == 0) error("Invalid UTF-8 sequence in '%s'", s);
 			tmp += len;
 		}
 	}
 
-	// Check if the string has a case..
-	// The syntax for cases is IDENTNAME.case
+	/* Check if the string has a case..
+	 * The syntax for cases is IDENTNAME.case */
 	casep = strchr(str, '.');
 	if (casep) *casep++ = 0;
 
-	// Check if this string already exists..
+	/* Check if this string already exists.. */
 	ent = HashFind(str);
 
 	if (master) {
 		if (ent != NULL && casep == NULL) {
-			error("String name '%s' is used multiple times", str);
+			strgen_error("String name '%s' is used multiple times", str);
 			return;
 		}
 
 		if (ent == NULL && casep != NULL) {
-			error("Base string name '%s' doesn't exist yet. Define it before defining a case.", str);
+			strgen_error("Base string name '%s' doesn't exist yet. Define it before defining a case.", str);
 			return;
 		}
 
 		if (ent == NULL) {
 			if (_strings[_next_string_id]) {
-				error("String ID 0x%X for '%s' already in use by '%s'", ent, str, _strings[_next_string_id]->name);
+				strgen_error("String ID 0x%X for '%s' already in use by '%s'", _next_string_id, str, _strings[_next_string_id]->name);
 				return;
 			}
 
-			// Allocate a new LangString
+			/* Allocate a new LangString */
 			ent = CallocT<LangString>(1);
 			_strings[_next_string_id] = ent;
 			ent->index = _next_string_id++;
@@ -847,7 +882,7 @@ static void HandleString(char *str, bool master)
 		}
 
 		if (casep != NULL) {
-			Case* c = MallocT<Case>(1);
+			Case *c = MallocT<Case>(1);
 
 			c->caseidx = ResolveCaseName(casep, strlen(casep));
 			c->string = strdup(s);
@@ -859,24 +894,24 @@ static void HandleString(char *str, bool master)
 
 	} else {
 		if (ent == NULL) {
-			warning("String name '%s' does not exist in master file", str);
+			strgen_warning("String name '%s' does not exist in master file", str);
 			return;
 		}
 
 		if (ent->translated && casep == NULL) {
-			error("String name '%s' is used multiple times", str);
+			strgen_error("String name '%s' is used multiple times", str);
 			return;
 		}
 
 		if (s[0] == ':' && s[1] == '\0' && casep == NULL) {
-			// Special syntax :: means we should just inherit the master string
+			/* Special syntax :: means we should just inherit the master string */
 			ent->translated = strdup(ent->english);
 		} else {
-			// make sure that the commands match
+			/* make sure that the commands match */
 			if (!CheckCommandsMatch(s, ent->english, str)) return;
 
 			if (casep != NULL) {
-				Case* c = MallocT<Case>(1);
+				Case *c = MallocT<Case>(1);
 
 				c->caseidx = ResolveCaseName(casep, strlen(casep));
 				c->string = strdup(s);
@@ -908,11 +943,14 @@ static void ParseFile(const char *file, bool english)
 	/* For each new file we parse, reset the genders, and language codes */
 	_numgenders = 0;
 	_lang_name[0] = _lang_ownname[0] = _lang_isocode[0] = '\0';
-	// TODO:!! We can't reset the cases. In case the translated strings
-	// derive some strings from english....
+	_lang_textdir = TD_LTR;
+	_lang_winlangid = 0x0000; // neutral language code
+	_lang_newgrflangid = 0; // standard english
+	/* TODO:!! We can't reset the cases. In case the translated strings
+	 * derive some strings from english.... */
 
 	in = fopen(file, "r");
-	if (in == NULL) fatal("Cannot open file");
+	if (in == NULL) error("Cannot open file");
 	_cur_line = 1;
 	while (fgets(buf, sizeof(buf), in) != NULL) {
 		rstrip(buf);
@@ -922,7 +960,7 @@ static void ParseFile(const char *file, bool english)
 	fclose(in);
 
 	if (StrEmpty(_lang_name) || StrEmpty(_lang_ownname) || StrEmpty(_lang_isocode)) {
-		fatal("Language must include ##name, ##ownname and ##isocode");
+		error("Language must include ##name, ##ownname and ##isocode");
 	}
 }
 
@@ -937,18 +975,18 @@ static uint32 MyHashStr(uint32 hash, const char *s)
 }
 
 
-// make a hash of the file to get a unique "version number"
+/* make a hash of the file to get a unique "version number" */
 static void MakeHashOfStrings()
 {
 	uint32 hash = 0;
 	uint i;
 
 	for (i = 0; i != lengthof(_strings); i++) {
-		const LangString* ls = _strings[i];
+		const LangString *ls = _strings[i];
 
 		if (ls != NULL) {
-			const CmdStruct* cs;
-			const char* s;
+			const CmdStruct *cs;
+			const char *s;
 			char buf[256];
 			int argno;
 			int casei;
@@ -991,7 +1029,7 @@ bool CompareFiles(const char *n1, const char *n2)
 	if (f2 == NULL) return false;
 
 	f1 = fopen(n1, "rb");
-	if (f1 == NULL) fatal("can't open %s", n1);
+	if (f1 == NULL) error("can't open %s", n1);
 
 	do {
 		l1 = fread(b1, 1, sizeof(b1), f1);
@@ -1017,7 +1055,7 @@ static void WriteStringsH(const char *filename)
 	int next = -1;
 
 	out = fopen("tmp.xxx", "w");
-	if (out == NULL) fatal("can't open tmp.xxx");
+	if (out == NULL) error("can't open tmp.xxx");
 
 	fprintf(out, "/* This file is automatically generated. Do not modify */\n\n");
 	fprintf(out, "#ifndef TABLE_STRINGS_H\n");
@@ -1045,14 +1083,14 @@ static void WriteStringsH(const char *filename)
 	fclose(out);
 
 	if (CompareFiles("tmp.xxx", filename)) {
-		// files are equal. tmp.xxx is not needed
+		/* files are equal. tmp.xxx is not needed */
 		unlink("tmp.xxx");
 	} else {
-		// else rename tmp.xxx into filename
+		/* else rename tmp.xxx into filename */
 #if defined(WIN32) || defined(WIN64)
 		unlink(filename);
 #endif
-		if (rename("tmp.xxx", filename) == -1) fatal("rename() failed");
+		if (rename("tmp.xxx", filename) == -1) error("rename() failed");
 	}
 }
 
@@ -1060,8 +1098,8 @@ static int TranslateArgumentIdx(int argidx)
 {
 	int i, sum;
 
-	if (argidx < 0 || argidx >= lengthof(_cur_pcs.cmd))
-		fatal("invalid argidx %d", argidx);
+	if (argidx < 0 || (uint)argidx >= lengthof(_cur_pcs.cmd))
+		error("invalid argidx %d", argidx);
 
 	for (i = sum = 0; i < argidx; i++) {
 		const CmdStruct *cs = _cur_pcs.cmd[i];
@@ -1088,7 +1126,7 @@ static void PutCommandString(const char *str)
 	_cur_argidx = 0;
 
 	while (*str != '\0') {
-		// Process characters as they are until we encounter a {
+		/* Process characters as they are until we encounter a { */
 		if (*str != '{') {
 			PutByte(*str++);
 			continue;
@@ -1101,18 +1139,18 @@ static void PutCommandString(const char *str)
 			PutByte(casei);
 		}
 
-		// For params that consume values, we need to handle the argindex properly
+		/* For params that consume values, we need to handle the argindex properly */
 		if (cs->consumes > 0) {
-			// Check if we need to output a move-param command
+			/* Check if we need to output a move-param command */
 			if (argno != -1 && argno != _cur_argidx) {
 				_cur_argidx = argno;
 				PutArgidxCommand();
 			}
 
-			// Output the one from the master string... it's always accurate.
+			/* Output the one from the master string... it's always accurate. */
 			cs = _cur_pcs.cmd[_cur_argidx++];
 			if (cs == NULL) {
-				fatal("%s: No argument exists at position %d", _cur_ident, _cur_argidx - 1);
+				error("%s: No argument exists at position %d", _cur_ident, _cur_argidx - 1);
 			}
 		}
 
@@ -1128,7 +1166,7 @@ static void WriteLength(FILE *f, uint length)
 		fputc((length >> 8) | 0xC0, f);
 		fputc(length & 0xFF, f);
 	} else {
-		fatal("string too long");
+		error("string too long");
 	}
 }
 
@@ -1142,7 +1180,7 @@ static void WriteLangfile(const char *filename)
 	uint j;
 
 	f = fopen(filename, "wb");
-	if (f == NULL) fatal("can't open %s", filename);
+	if (f == NULL) error("can't open %s", filename);
 
 	memset(&hdr, 0, sizeof(hdr));
 	for (i = 0; i != 32; i++) {
@@ -1152,23 +1190,26 @@ static void WriteLangfile(const char *filename)
 		hdr.offsets[i] = TO_LE16(n);
 	}
 
-	// see line 655: fprintf(..."\tLANGUAGE_PACK_IDENT = 0x474E414C,...)
+	/* see line 655: fprintf(..."\tLANGUAGE_PACK_IDENT = 0x474E414C,...) */
 	hdr.ident = TO_LE32(0x474E414C); // Big Endian value for 'LANG'
 	hdr.version = TO_LE32(_hash);
 	hdr.plural_form = _lang_pluralform;
-	strcpy(hdr.name, _lang_name);
-	strcpy(hdr.own_name, _lang_ownname);
-	strcpy(hdr.isocode, _lang_isocode);
+	hdr.text_dir = _lang_textdir;
+	hdr.winlangid = TO_LE16(_lang_winlangid);
+	hdr.newgrflangid = _lang_newgrflangid;
+	strecpy(hdr.name, _lang_name, lastof(hdr.name));
+	strecpy(hdr.own_name, _lang_ownname, lastof(hdr.own_name));
+	strecpy(hdr.isocode, _lang_isocode, lastof(hdr.isocode));
 
 	fwrite(&hdr, sizeof(hdr), 1, f);
 
 	for (i = 0; i != 32; i++) {
 		for (j = 0; j != in_use[i]; j++) {
-			const LangString* ls = _strings[(i << 11) + j];
-			const Case* casep;
-			const char* cmdp;
+			const LangString *ls = _strings[(i << 11) + j];
+			const Case *casep;
+			const char *cmdp;
 
-			// For undefined strings, just set that it's an empty string
+			/* For undefined strings, just set that it's an empty string */
 			if (ls == NULL) {
 				WriteLength(f, 0);
 				continue;
@@ -1177,10 +1218,10 @@ static void WriteLangfile(const char *filename)
 			_cur_ident = ls->name;
 			_cur_line = ls->line;
 
-			// Produce a message if a string doesn't have a translation.
+			/* Produce a message if a string doesn't have a translation. */
 			if (_show_todo > 0 && ls->translated == NULL) {
 				if ((_show_todo & 2) != 0) {
-					warning("'%s' is untranslated", ls->name);
+					strgen_warning("'%s' is untranslated", ls->name);
 				}
 				if ((_show_todo & 1) != 0) {
 					const char *s = "<TODO> ";
@@ -1188,7 +1229,7 @@ static void WriteLangfile(const char *filename)
 				}
 			}
 
-			// Extract the strings and stuff from the english command string
+			/* Extract the strings and stuff from the english command string */
 			ExtractCommandString(&_cur_pcs, ls->english, false);
 
 			if (ls->translated_case != NULL || ls->translated != NULL) {
@@ -1202,31 +1243,31 @@ static void WriteLangfile(const char *filename)
 			_translated = _masterlang || (cmdp != ls->english);
 
 			if (casep != NULL) {
-				const Case* c;
+				const Case *c;
 				uint num;
 
-				// Need to output a case-switch.
-				// It has this format
-				// <0x9E> <NUM CASES> <CASE1> <LEN1> <STRING1> <CASE2> <LEN2> <STRING2> <CASE3> <LEN3> <STRING3> <STRINGDEFAULT>
-				// Each LEN is printed using 2 bytes in big endian order.
+				/* Need to output a case-switch.
+				 * It has this format
+				 * <0x9E> <NUM CASES> <CASE1> <LEN1> <STRING1> <CASE2> <LEN2> <STRING2> <CASE3> <LEN3> <STRING3> <STRINGDEFAULT>
+				 * Each LEN is printed using 2 bytes in big endian order. */
 				PutUtf8(SCC_SWITCH_CASE);
-				// Count the number of cases
+				/* Count the number of cases */
 				for (num = 0, c = casep; c; c = c->next) num++;
 				PutByte(num);
 
-				// Write each case
+				/* Write each case */
 				for (c = casep; c != NULL; c = c->next) {
 					int pos;
 
 					PutByte(c->caseidx);
-					// Make some space for the 16-bit length
+					/* Make some space for the 16-bit length */
 					pos = _put_pos;
 					PutByte(0);
 					PutByte(0);
-					// Write string
+					/* Write string */
 					PutCommandString(c->string);
 					PutByte(0); // terminate with a zero
-					// Fill in the length
+					/* Fill in the length */
 					_put_buf[pos + 0] = GB(_put_pos - (pos + 2), 8, 8);
 					_put_buf[pos + 1] = GB(_put_pos - (pos + 2), 0, 8);
 				}
@@ -1285,15 +1326,15 @@ static inline char *replace_pathsep(char *s)
 static inline char *replace_pathsep(char *s) { return s; }
 #endif
 
-int CDECL main(int argc, char* argv[])
+int CDECL main(int argc, char *argv[])
 {
-	char pathbuf[256];
+	char pathbuf[MAX_PATH];
 	const char *src_dir = ".";
 	const char *dest_dir = NULL;
 
 	while (argc > 1 && *argv[1] == '-') {
 		if (strcmp(argv[1], "-v") == 0 || strcmp(argv[1], "--version") == 0) {
-			puts("$Revision: 12478 $");
+			puts("$Revision: 16830 $");
 			return 0;
 		}
 
@@ -1311,7 +1352,7 @@ int CDECL main(int argc, char* argv[])
 
 		if (strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "--help") == 0 || strcmp(argv[1], "-?") == 0) {
 			puts(
-				"strgen - $Revision: 12478 $\n"
+				"strgen - $Revision: 16830 $\n"
 				" -v | --version    print version information and exit\n"
 				" -t | --todo       replace any untranslated strings with '<TODO>'\n"
 				" -w | --warning    print a warning for any untranslated strings\n"
