@@ -1,4 +1,4 @@
-/* $Id: vehicle_base.h 16482 2009-05-31 12:22:53Z rubidium $ */
+/* $Id: vehicle_base.h 16742 2009-07-04 17:39:00Z rubidium $ */
 
 /** @file  vehicle_base.h Base class for all vehicles. */
 
@@ -223,7 +223,9 @@ public:
 
 	Money profit_this_year;        ///< Profit this year << 8, low 8 bits are fract
 	Money profit_last_year;        ///< Profit last year << 8, low 8 bits are fract
-	Money value;
+	Money value;                   ///< Value of the vehicle
+
+	CargoPayment *cargo_payment;   ///< The cargo payment we're currently in
 
 	/* Used for timetabling. */
 	uint32 current_order_time;     ///< How many ticks have passed since this order started.
